=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2008-06-17 03:55:49 +0000
+++ bzrlib/builtins.py	2008-06-24 00:48:51 +0000
@@ -134,7 +134,7 @@
     
     Note that --short or -S gives status flags for each item, similar
     to Subversion's status command. To get output similar to svn -q,
-    use bzr -SV.
+    use bzr status -SV.
 
     If no arguments are specified, the status of the entire working
     directory is shown.  Otherwise, only the status of the specified

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2008-05-29 19:46:01 +0000
+++ bzrlib/merge.py	2008-06-23 20:12:16 +0000
@@ -419,15 +419,14 @@
             merge = self.make_merger()
             merge.do_merge()
             if self.recurse == 'down':
-                for path, file_id in self.this_tree.iter_references():
-                    sub_tree = self.this_tree.get_nested_tree(file_id, path)
+                for relpath, file_id in self.this_tree.iter_references():
+                    sub_tree = self.this_tree.get_nested_tree(file_id, relpath)
                     other_revision = self.other_tree.get_reference_revision(
-                        file_id, path)
+                        file_id, relpath)
                     if  other_revision == sub_tree.last_revision():
                         continue
                     sub_merge = Merger(sub_tree.branch, this_tree=sub_tree)
                     sub_merge.merge_type = self.merge_type
-                    relpath = self.this_tree.relpath(path)
                     other_branch = self.other_branch.reference_parent(file_id, relpath)
                     sub_merge.set_other_revision(other_revision, other_branch)
                     base_revision = self.base_tree.get_reference_revision(file_id)

=== modified file 'bzrlib/tests/tree_implementations/__init__.py'
--- bzrlib/tests/tree_implementations/__init__.py	2008-06-24 18:04:00 +0000
+++ bzrlib/tests/tree_implementations/__init__.py	2008-06-24 19:45:27 +0000
@@ -69,7 +69,7 @@
 
 
 def _dirstate_tree_from_workingtree(testcase, tree):
-    revid = tree.commit('save tree', allow_pointless=True)
+    revid = tree.commit('save tree', allow_pointless=True, recursive=None)
     return tree.basis_tree()
 
 
@@ -349,7 +349,7 @@
             to be used as members of the TestCase.
         """
         if workingtree_format is None:
-            workingtree_format = WorkingTreeFormat3()
+            workingtree_format = WorkingTreeFormat._default_format
         scenario_options = WorkingTreeTestProviderAdapter.create_scenario(self,
             workingtree_format, workingtree_format._matchingbzrdir)[1]
         scenario_options["_workingtree_to_test_tree"] = converter

=== modified file 'bzrlib/tests/tree_implementations/test_tree.py'
--- bzrlib/tests/tree_implementations/test_tree.py	2008-01-03 22:14:40 +0000
+++ bzrlib/tests/tree_implementations/test_tree.py	2008-06-20 19:06:38 +0000
@@ -69,7 +69,7 @@
 
     def skip_if_no_reference(self, tree):
         if not getattr(tree, 'supports_tree_reference', lambda: False)():
-            raise tests.TestSkipped('Tree references not supported')
+            raise tests.TestNotApplicable('Tree references not supported')
 
     def create_nested(self):
         work_tree = self.make_branch_and_tree('wt')
@@ -88,6 +88,8 @@
 
     def test_get_reference_revision(self):
         tree = self.create_nested()
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
         path = tree.id2path('sub-root')
         self.assertEqual('sub-1', tree.get_reference_revision('sub-root', path))
 
@@ -96,7 +98,7 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         entry = tree.inventory['sub-root']
-        self.assertEqual([(tree.abspath('subtree'), 'sub-root')],
+        self.assertEqual([(u'subtree', 'sub-root')],
             list(tree.iter_references()))
 
     def test_get_root_id(self):

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2008-06-24 18:04:00 +0000
+++ bzrlib/transform.py	2008-06-24 19:45:27 +0000
@@ -1402,7 +1402,8 @@
         except KeyError:
             return
         file_id = self.tree_file_id(parent_id)
-        for child in self._tree.inventory[file_id].children.iterkeys():
+        children = getattr(self._tree.inventory[file_id], 'children', {})
+        for child in children:
             childpath = joinpath(path, child)
             yield self.trans_id_tree_path(childpath)
 
@@ -1582,13 +1583,15 @@
         while len(todo) > 0:
             parent = todo.pop()
             parent_file_id = self._transform.final_file_id(parent)
-            children = self._all_children(parent)
+            children = list(self._all_children(parent))
+            paths = dict(zip(children, self._final_paths.get_paths(children)))
+            children.sort(key=paths.get)
             todo.extend(children)
             for trans_id in children:
                 ordered_ids.append((trans_id, parent_file_id))
         for entry, trans_id in self._make_inv_entries(ordered_ids,
                                                       specific_file_ids):
-            yield self._final_paths.get_path(trans_id), entry
+            yield unicode(self._final_paths.get_path(trans_id)), entry
 
     def kind(self, file_id):
         trans_id = self._transform.trans_id_file_id(file_id)

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2008-05-08 04:33:38 +0000
+++ bzrlib/tree.py	2008-06-24 18:54:58 +0000
@@ -176,10 +176,26 @@
     def iter_entries_by_dir(self, specific_file_ids=None):
         """Walk the tree in 'by_dir' order.
 
-        This will yield each entry in the tree as a (path, entry) tuple. The
-        order that they are yielded is: the contents of a directory are 
-        preceeded by the parent of a directory, and all the contents of a 
-        directory are grouped together.
+        This will yield each entry in the tree as a (path, entry) tuple.
+        The order that they are yielded is:
+
+        Directories are walked in a depth-first lexicographical order,
+        however, whenever a directory is reached, all of its direct child
+        nodes are yielded in  lexicographical order before yielding the
+        grandchildren.
+
+        For example, in the tree::
+
+           a/
+             b/
+               c
+             d/
+               e
+           f/
+             g
+
+        The yield order would be::
+          a, f, a/b, a/d, a/b/c, a/d/e, f/g
         """
         return self.inventory.iter_entries_by_dir(
             specific_file_ids=specific_file_ids)

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2008-06-06 16:40:46 +0000
+++ bzrlib/workingtree_4.py	2008-06-20 19:06:38 +0000
@@ -528,6 +528,10 @@
         return iter(result)
 
     def iter_references(self):
+        if not self._repo_supports_tree_reference:
+            # When the repo doesn't support references, we will have nothing to
+            # return
+            return
         for key, tree_details in self.current_dirstate()._iter_entries():
             if tree_details[0][0] in ('a', 'r'): # absent, relocated
                 # not relevant to the working tree
@@ -535,10 +539,10 @@
             if not key[1]:
                 # the root is not a reference.
                 continue
-            path = pathjoin(self.basedir, key[0].decode('utf8'), key[1].decode('utf8'))
+            relpath = pathjoin(key[0].decode('utf8'), key[1].decode('utf8'))
             try:
-                if self._kind(path) == 'tree-reference':
-                    yield path, key[2]
+                if self._kind(relpath) == 'tree-reference':
+                    yield relpath, key[2]
             except errors.NoSuchFile:
                 # path is missing on disk.
                 continue
@@ -1415,6 +1419,9 @@
         self._inventory = None
         self._locked = 0
         self._dirstate_locked = False
+        self._repo_supports_tree_reference = getattr(
+            repository._format, "supports_tree_reference",
+            False)
 
     def __repr__(self):
         return "<%s of %s in %s>" % \
@@ -1459,6 +1466,14 @@
         path_utf8 = osutils.pathjoin(entry[0][0], entry[0][1])
         return path_utf8.decode('utf8')
 
+    def iter_references(self):
+        if not self._repo_supports_tree_reference:
+            # When the repo doesn't support references, we will have nothing to
+            # return
+            return iter([])
+        # Otherwise, fall back to the default implementation
+        return super(DirStateRevisionTree, self).iter_references()
+
     def _get_parent_index(self):
         """Return the index in the dirstate referenced by this tree."""
         return self._dirstate.get_parent_ids().index(self._revision_id) + 1
@@ -1725,6 +1740,10 @@
                 self._dirstate_locked = False
             self._repository.unlock()
 
+    @needs_read_lock
+    def supports_tree_reference(self):
+        return self._repo_supports_tree_reference
+
     def walkdirs(self, prefix=""):
         # TODO: jam 20070215 This is the lazy way by using the RevisionTree
         # implementation based on an inventory.  

